
//-----------------Layer-----------------
//Abstract object for InDesign Layer

//param: layer is native InDesign layer, layerIndex is the layer stack position
EFI.Layer = function(layer, layerIndex) {
	this.layer = layer;
	this.layerIndex = layerIndex; //layer stack position
	this.id = layer.id; //InDesign Id
	this.initiallyVisible = layer.visible;
	this.variableObjectIndexes = [];
	this.objectsMadeHidden = [];
	this.layerSegments = [];

	this.populateVariableObjectIndexes();
};

EFI.Layer.prototype.getId = function() {
	return this.id;
};

EFI.Layer.prototype.getLayerSegments = function() {
	return this.layerSegments;
};

EFI.Layer.prototype.hide = function() {
	this.layer.visible = false;
};

EFI.Layer.prototype.makeOriginalVisible = function() {
	if(this.initiallyVisible) this.layer.visible = true;
};

EFI.Layer.prototype.getPageItems = function() {
	//return this.layer.allPageItems.everyItem().getElements().slice(0).reverse();
	return this.layer.allPageItems.slice(0).reverse();
};

EFI.Layer.prototype.getLastPageItemsIndex = function() {
	return this.layer.pageItems.length - 1;
};

EFI.Layer.prototype.isVisible = function() {
	return this.layer.visible;
};

EFI.Layer.prototype.hasVariableObjects = function() {
	return this.variableObjectIndexes.length > 0;
};

EFI.Layer.prototype.getVariableObjectCount = function() {
	return this.variableObjectIndexes.length;
};

EFI.Layer.prototype.addVariableObjectIndex = function(index) {
	this.variableObjectIndexes.push(index);
};

EFI.Layer.prototype.addLayerSegment = function(startIndex, endIndex, selectableObject) {
	//alert("adding layer " + this.layerIndex + selectableObject + " " + startIndex + " " + endIndex);
	var layerSegment = new EFI.LayerSegment(this, selectableObject, startIndex, endIndex);
	this.layerSegments.push(layerSegment);
};

EFI.Layer.prototype.getVariableObjectIndex = function(index) {
	if(index > this.variableObjectIndexes.length - 1) {
		return this.variableObjectIndexes[this.variableObjectIndexes.length - 1];
	} else {
		return this.variableObjectIndexes[index];
	}
};

EFI.Layer.prototype.isVariableObject = function(selId) {
		var currentDocument = EFI.UIHelper.getCurrentDocument();
		var boxObject = "";
		var blnReturn = false;
		var strFieldValue = "";
		var labelName="EFISmartCanvasExporter";

		if(currentDocument.extractLabel(labelName) !== "") {
			boxObject = JSON.parse(currentDocument.extractLabel(labelName));
			for(var i=0; i < boxObject.boxProperties.length; i++) {
				if(boxObject.boxProperties[i].id === selId) {
					strFieldValue = boxObject.boxProperties[i].boxVariable.fieldValue;
					if(strFieldValue === false) {
						blnReturn = false;
					} else {
						blnReturn = true;
					}
				}
			}
		}
		return blnReturn;
};

//break the layer up into static segments and variable segments
EFI.Layer.prototype.populateVariableObjectIndexes = function() {
	var pageItems =  this.getPageItems();
	var startSegmentIndex = 0; //starting index of the segment currently being worked on
	var endSegmentIndex = 0; //the ending index of the previous segment
	var fieldManager = EFI.ObjectFactory.getInstance("fieldManager");

	//look items in Layer and check if any are Variable
	for (var i = 0; i < pageItems.length; i++) {
		variableItem = fieldManager.getVariableObject(pageItems[i].id);
		if (variableItem !== undefined) {
			if (variableItem) {
				//if (variableItem.objectType == EFI.ConfigSettings.objectTypes.textframe) {
				//AK - if text frame and variable process contents

				if (this.isVariableObject(pageItems[i].id) === true) {
					this.addVariableObjectIndex(i);
					endSegmentIndex = i - 1;

					//add previous static segment as we hit a Variable field
					if (startSegmentIndex !== i) {
						this.addLayerSegment(startSegmentIndex, endSegmentIndex);
					}

					startSegmentIndex = i + 1;
					//add Variable field as a segment
					this.addLayerSegment(startSegmentIndex, startSegmentIndex, variableItem);
				}
				//}
			}
		} 
	}

	if(endSegmentIndex !== pageItems.length - 1 && endSegmentIndex !== -1) {
		this.addLayerSegment(startSegmentIndex, pageItems.length - 1);
	}
    
    if(this.layerSegments.length === 0 && pageItems.length > 0){
        this.addLayerSegment (0, pageItems.length - 1);
    }
};

//Create PDFs for static elements, assume all other layers are already hidden
EFI.Layer.prototype.generateStaticPDFs = function (exportFolder, profile) {
	var exportFile = undefined;

	if(this.getPageItems().length > this.getVariableObjectCount()) {		
		var pdfExporter = new EFI.PDFEngine();

		if(this.hasVariableObjects()) {
			//loop layer segments and generate PDFs
			for (var i = 0; i < this.layerSegments.length; i++) {
				var layerSegment = this.layerSegments[i];

				if(layerSegment.isStatic()) {
					this.makeVisibleObjectRange(layerSegment.startIndex, layerSegment.endIndex);

					exportFile = File(exportFolder + "/layer" + this.layerIndex + "-" + i + ".pdf");
					pdfExporter.exportPDF(exportFolder, exportFile, profile);
					layerSegment.PDFFile = exportFile;
				}
			}

			this.makeObjectsVisible();
		} else {
			exportFile = File(exportFolder + "/layer" + this.layerIndex + ".pdf");
			pdfExporter.exportPDF(exportFolder, exportFile, profile);
			this.layerSegments[0].PDFFile = exportFile;
		}
	}
};

EFI.Layer.prototype.makeObjectsVisible = function() {
	for(var i = 0; i < this.objectsMadeHidden.length; i++) this.objectsMadeHidden[i].visible = true;

	this.objectsMadeHidden = [];
};

EFI.Layer.prototype.makeVisibleObjectRange = function(startIndex, endIndex) {
	this.makeObjectsVisible();
	var pageItems = this.getPageItems();

	//hide fields outside of the range
	for(var i = 0; i < pageItems.length; i++) {
		if(i < startIndex || i > endIndex) {
			var pageItem = pageItems[i];

			//if it is visible, hide and add to hidden array
			if(pageItem.visible) {
				pageItems[i].visible = false;
				this.objectsMadeHidden.push(pageItems[i]);
			}
		}
	}
};
//-----------------//Layer-----------------