﻿
//----------------RuleEngine-------------
//Hands the building of rules/relationships between fields

//generates the Field rules from a InDesign TextFrame
EFI.RuleEngine = function(textFrame, multiline) {
	this.multiline = multiline;
	this.textFrame = textFrame;

	this.selectableObject = EFI.ObjectFactory.getInstance("selectableObject", this.textFrame);
	this.textBlock = this.selectableObject.getTextBlock();
	this.anchorField = undefined;

	var fieldManager = EFI.ObjectFactory.getInstance("FieldManager");
	this.textOutputType = EFI.ConfigSettings.outputType;
};

/*
	TODO: Handle captions
	TODO: Handle separator characters
	TODO: Handle Tabs
	TODO: Handle rotations
 */
EFI.RuleEngine.prototype.processFieldRules = function() {
	EFI.Utils.addDebugLog("EFI.RuleEngine.prototype.processFieldRules: id: " + this.selectableObject.getId());

	this.populateTextBlock();
	this.calculateRules();

	return this.selectableObject;
};

EFI.RuleEngine.prototype.populateTextBlock = function() {
	var textBlock = this.textBlock;
	textBlock.clearFields();

	textBlock.verticalAlign = EFI.UIHelper.getVerticalAlign(textBlock.textFrame);
	EFI.Utils.addDebugLog("EFI.RuleEngine.prototype.populateTextBlock: Vertical Align - " + textBlock.verticalAlign);

	if(this.multiline) {
		foundFields = EFI.TextFormatHelper.searchForMultline(this.textFrame);

		this.buildFields(foundFields, 1);
	} else {
		for (var i = 0; i < textBlock.textFrame.lines.length; i++) {
			foundFields = EFI.TextFormatHelper.searchForFields(textBlock.textFrame.lines[i]);
			this.buildFields(foundFields, i + 1);
		}
	}
};

EFI.RuleEngine.prototype.buildFields = function(foundFields, lineNumber) {
	var textBlock = this.textBlock;
	var descriptionFields = "";
	var field; //"text" object of InDesign
	var prevField;

	if(foundFields.length > 0) {

		for(var i = 0; i < foundFields.length; i++) {
			field = this.createField(foundFields[i], prevField);
			prevField = field;
			textBlock.addField(field, lineNumber, true);
		}
	}
};

//TODO: Handle text properties
EFI.RuleEngine.prototype.createField = function(textObject, prevField, fieldNumber) {
	//if(textObject) EFI.Utils.displayObjectProperties(textObject);
	var field = new EFI.Field(this.selectableObject.getId());

	field.originalX = EFI.UIHelper.getXCoordinate(textObject);
	field.originalY = EFI.UIHelper.getYCoordinate(textObject);
	field.rotation = EFI.UIHelper.getRotation(textObject);
	field.placeHolder = textObject.contents;
	field.testData = field.placeHolder;
	field.prevField = prevField;
	field.font = EFI.TextFormatHelper.getFontName(textObject); // textObject.appliedFont.fullName;
	field.fontSize = EFI.TextFormatHelper.getFontSize(textObject); // textObject.pointSize;
	field.alignment = EFI.TextFormatHelper.getAlignment(textObject);
    
    if(textObject.fillColor &&  textObject.fillColor.hasOwnProperty ("colorValue")) {
        field.colorC = textObject.fillColor.colorValue[0];
        field.colorM = textObject.fillColor.colorValue[1];
        field.colorY = textObject.fillColor.colorValue[2];
        field.colorK = textObject.fillColor.colorValue[3];
        field.colorK = textObject.fillColor.colorValue[3];
    }

	field.layerId = EFI.UIHelper.getLayerId(textObject);
	field.multiline = this.multiline;

	if(this.multiline) {
		field.defaultText = EFI.TextFormatHelper.getTextFrameOutput(this.textFrame);
	} else {
		field.defaultText = this.textFrame.contents;
	}

	field.setTextObject(textObject);
	field.setFieldCharacterPosition();

	return field;
};

EFI.RuleEngine.prototype.calculateRules = function() {
	var textBlock = this.textBlock;
	if(textBlock.getNumberOfFields() > 0) {
		this.anchorField = this.getTextBlockAnchorField();
         
         if(this.anchorField){
            this.processField(this.anchorField.textBlockLine.getLastField());
         }
	}
};

EFI.RuleEngine.prototype.getTextBlockAnchorField = function() {
	var textBlock = this.textBlock;
	var anchorField = undefined;

	if (textBlock.getNumberOfFields() > 0) {
		switch (textBlock.verticalAlign) {
			//very first field
			case "top":
				anchorField = textBlock.getLine(1).getFirstField();
				break;
			//first field of last line
			case "bottom":
				anchorField = textBlock.getLine(textBlock.getNumberOfLines()).getFirstField();
				break;
			//very first field with y co-ordinate as center of block
			case "center":
				anchorField = textBlock.getLine(1).getFirstField();
				break;
		}
	}

	if(anchorField) {
		anchorField.isAnchorField = true;
		anchorField.verticalAlign = textBlock.verticalAlign;
	}

	return anchorField;
};

EFI.RuleEngine.prototype.processField = function(curField) {
	this.getCaption(curField);

	//assign x position for first field of the line for center/right
	if((curField.alignment === "center" || curField.alignment === "right") && curField.prevField === undefined) {
		curField.assignOriginalCoordinates(this.textBlock.verticalAlign);
		curField.y = undefined;
	}

	if(curField.isAnchorField) {
		curField.assignOriginalCoordinates(this.textBlock.verticalAlign);

		this.processNextLine(curField);
	} else if(curField.prevField) {
		//has a previous field so it is relative to it
		curField.relativeField = curField.prevField;
		curField.relativeDirection = "horizontal";

		this.checkSwitchCaptionFieldRelative(curField);

		//process previous fields recursively until end of line
		this.processField(curField.prevField);
	} else {
		//no relatives so must be first field, go to next line
		this.processNextLine(curField);
	}
};

EFI.RuleEngine.prototype.getCaption = function(curField) {
	var startCaptionPosition = 0;
	var endCaptionPosition = curField.getFieldStartCharacterPosition() - 1;
	var captionText = "";
	var captionSeparatorCharacter = "";
	var separatorCharacter = "";

	//if prev field, then caption start is end of previous field
	if(curField.prevField) {
		startCaptionPosition = curField.prevField.getFieldEndCharacterPosition();
	} else {
		//first field so caption start position is start of the line
		startCaptionPosition = curField.textBlockLine.getStartCharacterPosition();

		//check if field is at start of line, if yes then no caption
		if(startCaptionPosition === curField.getFieldStartCharacterPosition()) startCaptionPosition = -1;

	}

	//check if there is no caption between the fields
	if(startCaptionPosition === curField.getFieldStartCharacterPosition()) startCaptionPosition = -1;

	if(startCaptionPosition > -1) {
		captionText = curField.getTextFrame().contents.substring(startCaptionPosition, endCaptionPosition + 1);
	}

	if(captionText !== "") {
		this.setCaptionProperties(curField, captionText, startCaptionPosition, endCaptionPosition);
		captionText = this.fixTabPosition(curField, captionText, startCaptionPosition, endCaptionPosition);
	}

	//check for separator characters
	if(captionText === ", ") {
		captionText = "";
		separatorCharacter = ", ";
	} else if(captionText === " ") {
		captionText = "";
		separatorCharacter = " ";
	} else if(captionText.substring(0,2) === ", ") {
		captionSeparatorCharacter = ", ";
		captionText = captionText.substring(2, captionText.length);
	} else if(captionText.substring(0,1) === ",") {
		captionSeparatorCharacter = ",";
		captionText = captionText.substring(1, captionText.length);
	} else if(captionText.substring(0,1) === " ") {
		captionSeparatorCharacter = " ";
		captionText = captionText.substring(1, captionText.length);
	}

	if(captionText !== "") {
		curField.captionDisplayDisplay = true;
		curField.relativeToCaption = "Yes";
	}

	curField.captionDisplayCaption = captionText;
	curField.separatorCharacter = separatorCharacter;
	curField.captionSeparatorCharacter = captionSeparatorCharacter;
};

//if field relative to caption, need to swap field relative with caption
EFI.RuleEngine.prototype.checkSwitchCaptionFieldRelative = function(curField) {
	if(curField.relativeToCaption === "Yes") {
		curField.captionRelativeDirection = curField.relativeDirection;
		curField.relativeDirection = "horizontal";

		curField.captionRelativeField = curField.relativeField;
		curField.relativeField = undefined;

		curField.captionSeparatorSize = curField.separatorSize;
		curField.separatorSize = "";

		if(curField.captionTabbed || (curField.alignment === "center" || curField.alignment === "right")) curField.captionX = curField.x;
		if(!curField.fieldTabbed) curField.x = undefined;

		curField.captionY = curField.y;
		curField.y = undefined;
	}
};

EFI.RuleEngine.prototype.setCaptionProperties = function(curField, captionText, startCaptionPosition, endCaptionPosition) {
	var textObject = EFI.TextFormatHelper.searchForText(this.textBlock.textFrame.lines[curField.textBlockLine.lineNumber - 1], captionText, startCaptionPosition, endCaptionPosition);

	if(textObject) {
		curField.captionFont = EFI.TextFormatHelper.getFontName(textObject);
		curField.captionFontSize = EFI.TextFormatHelper.getFontSize(textObject);
		curField.captionAlignment = EFI.TextFormatHelper.getAlignment(textObject);
		curField.captionColorC = textObject.fillColor.colorValue[0];
		curField.captionColorM = textObject.fillColor.colorValue[1];
		curField.captionColorY = textObject.fillColor.colorValue[2];
		curField.captionColorK = textObject.fillColor.colorValue[3];
	}
};

//this is first field of line so setup vertical relative and go to next line
EFI.RuleEngine.prototype.processNextLine = function(curField) {
	var nextLineToProcess;
	var relativeField;

	switch (this.textBlock.verticalAlign) {
		case "top":
		case "center":
			var prevLine = curField.textBlockLine.getPreviousLine();
			if(prevLine) relativeField = prevLine.getFirstField();

			nextLineToProcess = curField.textBlockLine.getNextLine();
			break;
		case "bottom":
			var nextLine = curField.textBlockLine.getNextLine();
			if(nextLine) relativeField = nextLine.getFirstField();

			nextLineToProcess = curField.textBlockLine.getPreviousLine();
			break;
	}

	if (relativeField) {
		curField.relativeField = relativeField;
		curField.relativeDirection = "vertical";
		curField.separatorSize = this.getSeparatorSize(curField, relativeField);
	}

	this.checkSwitchCaptionFieldRelative(curField);

	//if there is another line then start processing with its last field
	if (nextLineToProcess) {
		this.processField(nextLineToProcess.getLastField());
	}
};

EFI.RuleEngine.prototype.getSeparatorSize = function (curField, relativeField) {
	var separatorSize;

	switch (curField.rotation) {
		case 0:
			separatorSize = curField.originalY - relativeField.originalY;
			break;
		case 180:
			separatorSize = relativeField.originalY - curField.originalY;
			break;
		case -90:
			separatorSize = relativeField.originalX - curField.originalX;
			break;
		case 90:
			separatorSize = curField.originalX - relativeField.originalX;
			break;
	}

	return separatorSize;
};

EFI.RuleEngine.prototype.fixTabPosition = function(curField, captionText, startCaptionPosition, endCaptionPosition) {
	var moreTabs = true;
	var numberOfTabs = 0;
	var newCaptionText = captionText;

	if(newCaptionText.indexOf("\t") > -1) {
		//sort out tabs before
		while(moreTabs && newCaptionText.length > numberOfTabs) {
			if(newCaptionText.substring(numberOfTabs, numberOfTabs + 1) === "\t") {
				numberOfTabs++;
			} else {
				moreTabs = false;
			}
		}

		if(numberOfTabs > 0){
			newCaptionText = newCaptionText.substring(numberOfTabs, newCaptionText.length);
			var textObject = EFI.TextFormatHelper.searchForText(this.textBlock.textFrame.lines[curField.textBlockLine.lineNumber - 1], newCaptionText, startCaptionPosition + numberOfTabs, endCaptionPosition);

			if(textObject) {
				var newX = EFI.UIHelper.getXCoordinate(textObject);

				curField.x = newX;
				if(newCaptionText !== "") curField.captionTabbed = true;
				//} else {
				//	curField.captionX = newX;
				//}
			}
		}

		//sort out tabs after
		moreTabs = true;
		numberOfTabs = 0;
		var startPosition = 0;

		while(moreTabs && newCaptionText.length > numberOfTabs) {
			startPosition = newCaptionText.length - numberOfTabs - 1;

			if(newCaptionText.substring(startPosition, startPosition + 1) === "\t") {
				numberOfTabs++;
			} else {
				moreTabs = false;
			}
		}

		if(numberOfTabs > 0){
			newCaptionText = newCaptionText.substring(0, newCaptionText.length - numberOfTabs);
			curField.x = curField.originalX;
			if(newCaptionText !== "") curField.fieldTabbed = true;
		}
	}

	return newCaptionText;
};
//----------------//RuleEngine-----------